from fastapi import Header, HTTPException, UploadFile
from contextlib import contextmanager
from erre2.database.db import SessionLocal
import aiofiles
import os


async def get_erre2_version():
    return "2.0.1"


async def get_auth_token(authorization: str = Header(...)):
    if authorization != "fuf":
        raise HTTPException(status_code=400, detail="tmp")


def get_db():
    with SessionLocal() as ses:
        yield ses

@contextmanager
def SessionManager():
    session = SessionLocal()
    yield session


async def save_file(file: UploadFile, summary):
    out_file_path = os.path.join("Files", "{}_".format(summary.sid)+file.filename)
    async with aiofiles.open(out_file_path, "wb") as out_file:
        content = await file.read()
        await out_file.write(content)
    return file.filename



import os

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker



SQLALCHEMY_DATABASE_URL = os.getenv("DB_URI")

DB_POOL_SIZE = 100,
WEB_CONCURRENCY = 2,
val = 100 // 2
POOL_SIZE = max(val, 5)
engine = create_engine(
    SQLALCHEMY_DATABASE_URL,
    pool_size=POOL_SIZE
)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)

Base = declarative_base()


from sqlalchemy import Integer, String, LargeBinary, Column, Boolean, ForeignKey, SmallInteger, DateTime
from sqlalchemy.orm import relationship
from erre2.database.schemas import User as UserSchema
from erre2.database.schemas import Course as CourseSchema
from erre2.database.schemas import Commit as CommitSchema

from erre2.database.db import Base


class User(Base):
    __tablename__ = "user"

    uid = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False)
    surname = Column(String, nullable=False)
    email = Column(String, nullable=False, unique=True)
    password = Column(LargeBinary, nullable=False)
    enabled = Column(Boolean, default=True)

    summaries = relationship("Summary", back_populates="author")
    server = relationship("Server", back_populates="owner")

    def to_schema(self):
        return UserSchema(uid=self.uid, name=self.name, surname=self.surname, email=self.email)


class Summary(Base):
    __tablename__ = "summary"

    sid = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False)
    filename = Column(String, nullable=False)
    downloads = Column(Integer, default=0)

    author = relationship("User", back_populates="summaries")
    course = relationship("Course", back_populates="docs")
    commits = relationship("Commit", back_populates="summary")

    author_id = Column(Integer, ForeignKey("user.uid"))
    course_id = Column(Integer, ForeignKey("course.cid"))


class Commit(Base):
    __tablename__ = "commit"

    cid = Column(Integer, primary_key=True, autoincrement=True)
    description = Column(String)
    date = Column(DateTime, nullable=False)

    summary = relationship("Summary", back_populates="commits")

    summary_id = Column(Integer, ForeignKey("summary.sid"))

    def to_schema(self):
        return CommitSchema(cid=self.cid, description=self.description, date=self.date, summary_id=self.summary_id)

class Course(Base):
    __tablename__ = "course"

    cid = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False)
    professor = Column(String, nullable=False)
    curriculum = Column(String, nullable=False)
    year = Column(Integer, nullable=False)
    semester = Column(Integer, nullable=False)
    docs = relationship("Summary", back_populates="course")

    def to_schema(self):
        return CourseSchema(cid=self.cid, name=self.name, professor=self.professor, curriculum=self.curriculum,
                            year=self.year, semester=self.semester)


class Server(Base):
    __tablename__ = "server"

    sid = Column(SmallInteger, primary_key=True)
    name = Column(String)
    university = Column(String, nullable=False)
    monetization_link = Column(String)
    motd = Column(String)

    owner = relationship("User", back_populates="server")

    owner_id = Column(Integer, ForeignKey("user.uid"))


from sqlalchemy.orm import Session
from datetime import datetime
from fastapi import UploadFile
import bcrypt

from erre2.database import schemas, models


def get_user(db: Session, uid: int):
    return db.query(models.User).filter(models.User.uid == uid).first()


def get_user_by_email(db: Session, email: str):
    return db.query(models.User).filter(models.User.email == email).first()


def get_users(db: Session):
    return db.query(models.User).all()


def create_user(db: Session, user: schemas.UserCreate):
    db_user = models.User(email=user.email, name=user.name, surname=user.surname, password=user.hash)
    db.add(db_user)
    db.commit()
    db.refresh(db_user)
    return db_user


def update_user(db: Session, user: schemas.UserCreate, uid: int):
    db_user = get_user(db, uid)
    if not db_user:
        return
    db_user.name = user.name
    db_user.surname = user.surname
    if user.hash:
        db_user.password = user.hash
    db_user.email = user.email
    db.commit()
    db.refresh(db_user)
    return db_user


def get_commit(db: Session, cid: int):
    return db.query(models.Commit).filter(models.Commit.cid == cid).first()


def get_commits(db: Session):
    return db.query(models.Summary).all()


def create_commit(db: Session, commit: schemas.Commit):
    if not get_summary(db, commit.summary_id):
        return
    db_commit = models.Commit(description=commit.description, date=commit.date, summary_id=commit.summary_id)
    db.add(db_commit)
    db.commit()
    db.refresh(db_commit)
    return db_commit


def get_course(db: Session, cid: int):
    return db.query(models.Course).filter(models.Course.cid == cid).first()


def get_courses(db: Session):
    return db.query(models.Course).all()


def create_course(db: Session, course: schemas.Course):
    db_course = models.Course(name=course.name, professor=course.professor, curriculum=course.curriculum,
                              year=course.year, semester=course.semester)
    db.add(db_course)
    db.commit()
    db.refresh(db_course)
    return db_course


def update_course(db: Session, course: schemas.Course, cid: int):
    db_course: models.Course = get_course(db, cid)
    if not db_course:
        return
    db_course.name = course.name
    db_course.professor = course.professor
    db_course.curriculum = course.curriculum
    db_course.year = course.year
    db_course.semester = course.semester
    db.commit()
    db.refresh(db_course)
    return db_course


def get_summary(db: Session, sid: int):
    return db.query(models.Summary).filter(models.Summary.sid == sid).first()


def get_summaries(db: Session):
    return db.query(models.Summary).all()


def get_summaries_course(db: Session, course_id: int):
    return db.query(models.Summary).filter(models.Summary.course_id == course_id).all()


def create_summary(db: Session, summary: schemas.Summary, file:UploadFile):
    if not get_course(db, summary.course_id):
        return
    db_summary = models.Summary(name=summary.name,
                                filename="tmp", downloads=0,
                                author_id=summary.author_id,
                                course_id=summary.course_id)
    db.add(db_summary)
    db.commit()
    db.refresh(db_summary)
    db_summary.filename = "{}_{}".format(db_summary.sid, file.filename)
    db.commit()
    db.refresh(db_summary)
    return db_summary


def update_summary(db: Session, summary: schemas.Summary, sid: int, description: str, file:UploadFile):
    db_summary = get_summary(db, sid)
    if not db_summary:
        return
    db_summary.filename = "{}_{}".format(summary.sid, file.filename)
    db.commit()
    create_commit(db, schemas.Commit(description=description, date=datetime.now(), summary_id=sid))
    db.refresh(db_summary)
    return db_summary


def remove_summary(db: Session, sid: int):
    db_summary: models.Summary = get_summary(db, sid)
    for commit in db_summary.commits:
        db.delete(commit)
    db.delete(db_summary)
    db.commit()


def get_server(db: Session):
    return db.query(models.Server).first()


def update_server(db: Session, update: schemas.Server):
    absent = False
    server: models.Server = get_server(db)
    if not server:
        server = models.Server()
        absent = True
    server.name = update.name
    server.university = update.university
    server.monetization_link = update.monetization_link
    server.motd = update.motd
    server.owner_id = update.owner_id
    if absent:
        db.add(server)
        db.commit()
    db.commit()
    db.refresh(server)
    return server



from fastapi import Depends, FastAPI, HTTPException, status
from fastapi.staticfiles import StaticFiles
import uvicorn
from fastapi.middleware.cors import CORSMiddleware
from datetime import timedelta

from erre2.routers import users, course, server, summary
from erre2.database import crud, schemas, models
from erre2.dependencies import get_db
from erre2.database.db import engine, SessionLocal
from erre2.authentication import Token, OAuth2PasswordRequestForm, authenticate_user, ACCESS_TOKEN_EXPIRE_MINUTES, \
    create_token, get_hash

models.Base.metadata.create_all(bind=engine)

app = FastAPI()

app.include_router(users.router)
app.include_router(course.router)
app.include_router(server.router)
app.include_router(summary.router)

origins = [
    "http://localhost.tiangolo.com",
    "https://localhost.tiangolo.com",
    "http://localhost",
    "http://localhost:8080",
    "http://localhost:3000",
    "http://localhost:5000",
    "https://navigator.erre2.fermitech.info"
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

app.mount("/files", StaticFiles(directory="Files"), name="files")

@app.get("/")
async def root():
    return {
        "message": "This is the backend of Erre2.0. If you see this page, it means that the server is alive and well."}


@app.post("/token", response_model=Token)
async def login(form_data: OAuth2PasswordRequestForm = Depends()):
    user = authenticate_user(form_data.username, form_data.password)
    if not user:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="Incorrect username or password",
                            headers={"WWW-Authenticate": "Bearer"})
    token = create_token(data={"sub": user.email})
    return {"access_token": token, "token_type": "bearer"}


if __name__ == "__main__":
    with SessionLocal() as db:
        if not db.query(models.User).filter_by(uid=1).first():
            crud.create_user(db, schemas.UserCreate(email="admin@admin.com", hash=get_hash("password"), name="admin",
                                                    surname="admin"))
    uvicorn.run(app, host="0.0.0.0", port=8000)



from fastapi.testclient import TestClient
from erre2.main import app

client = TestClient(app)


def login(username, password):
    response = client.post("/token", data={"username": username, "password": password})
    assert response.status_code == 200
    return response.json()


def test_connect():
    response = client.get("/")
    assert response.status_code == 200


def test_me():
    token = login("admin@admin.com", "password")
    response = client.get("users/me", headers={"Authorization": "Bearer " + token["access_token"]})
    assert response.status_code == 200


def test_create_user():
    token = login("admin@admin.com", "password")
    response = client.post("users/", headers={"Authorization": "Bearer " + token["access_token"]},
                           json={"name": "sgozzoli", "surname": "caione", "email": "pippo@paperino.it",
                                 "password": "password"})
    assert response.status_code == 200


def test_unauthorized_create():
    token = login("pippo@paperino.it", "password")
    response = client.post("users/", headers={"Authorization": "Bearer " + token["access_token"]},
                           json={"name": "sgozzoli", "surname": "caione", "email": "pippo@paperino.it",
                                 "password": "password"})
    assert response.status_code == 403



from fastapi import APIRouter, Depends, Request, HTTPException
from erre2.dependencies import get_auth_token, get_db, get_erre2_version
from erre2.authentication import get_current_user, check_admin
from erre2.database.crud import get_server, update_server
from sqlalchemy.orm import Session
from erre2.database import schemas, models
from typing import Optional
import bcrypt

router = APIRouter(
    prefix="/server",
    tags=["server"],
    responses={404: {"description": "Not found"}}
)


@router.get("/", tags=["server"], response_model=schemas.Server)
async def read_server(request: Request, db: Session = Depends(get_db)):
    """
    Gets current state of server
    """
    s: models.Server = get_server(db)
    return schemas.Server(name=s.name, university=s.university, monetization_link=s.monetization_link, motd=s.motd,
                          owner_id=s.owner_id,
                          owner=s.owner.to_schema())


@router.patch("/", tags=["server"], response_model=schemas.Server)
async def patch_server(request: Request, server: schemas.Server, db: Session = Depends(get_db),
                       current_user: models.User = Depends(get_current_user)):
    """
    Updates the state of the server
    """
    if not check_admin(current_user):
        raise HTTPException(403, "You are not authorized.")
    s: models.Server = update_server(db, server)
    if s:
        return schemas.Server(name=s.name, university=s.university, monetization_link=s.monetization_link, motd=s.motd,
                              owner_id=s.owner_id)


@router.get("/planetarium", tags=["server"], response_model=schemas.Planetarium)
async def planetarium_retrieve(version=Depends(get_erre2_version), db: Session = Depends(get_db)):
    """
    Responds to the planetarium master server
    """
    s = get_server(db)
    if s:
        return schemas.Planetarium(version=version, type="Erre2",
                                   server=schemas.Server(name=s.name, university=s.university,
                                                         monetization_link=s.monetization_link,
                                                         motd=s.motd, owner_id=s.owner_id,
                                                         owner=s.owner.to_schema()))



from fastapi import APIRouter, Depends, Request, HTTPException

from erre2.database.models import Server
from erre2.dependencies import get_auth_token, get_db
from erre2.authentication import get_current_user, check_admin
from erre2.database.crud import get_users, create_user, get_user, update_user, get_server
from sqlalchemy.orm import Session
from erre2.database import schemas, models
from typing import Optional
import bcrypt

router = APIRouter(
    prefix="/users",
    tags=["users"],
    responses={404: {"description": "Not found"}}
)


@router.get("/me", tags=["users"], response_model=schemas.User)
async def read_users_me(current_user: models.User = Depends(get_current_user)):
    """
    Returns data about the current user.
    """
    return current_user


@router.get("/", tags=["users"], response_model=schemas.UserList)
async def read_users_list(request: Request, user_id: Optional[int] = None, db: Session = Depends(get_db),
                          current_user: models.User = Depends(get_current_user)):
    """
    Returns list of all users, or just one.
    """
    if not check_admin(current_user):
        raise HTTPException(403, "You are not authorized.")
    if not user_id:
        users = get_users(db)
        return schemas.UserList(users=users)
    else:
        user = get_user(db, user_id)
        return schemas.UserList(users=[user, ])


@router.post("/", tags=["users"], response_model=schemas.User)
async def create_user_(request: Request, user: schemas.UserCreatePlain, db: Session = Depends(get_db),
                       current_user: models.User = Depends(get_current_user)):
    """
    Allows the creation of a new user
    """
    if not check_admin(current_user):
        raise HTTPException(403, "You are not authorized.")
    h = bcrypt.hashpw(bytes(user.password, encoding="utf-8"), bcrypt.gensalt())
    user: models.User = create_user(db,
                                    schemas.UserCreate(name=user.name, surname=user.surname,
                                                       email=user.email, hash=h))
    if user:
        return schemas.User(uid=user.uid, name=user.name, surname=user.surname, email=user.email)


@router.patch("/{user_id}", tags=["users"], response_model=schemas.User)
async def update_user_(user: schemas.UserCreatePlain, user_id: int, db: Session = Depends(get_db),
                       current_user: models.User = Depends(get_current_user)):
    """
    Allows user update
    """
    server: Server = get_server(db)
    if user_id != current_user.uid or server.owner_id != user_id:
        raise HTTPException(status_code=403, detail="You cannot edit other users.")
    h = bcrypt.hashpw(bytes(user.password, encoding="utf-8"), bcrypt.gensalt())
    if user.password == " ":
        h = None
    user_n = update_user(db, schemas.UserCreate(name=user.name, surname=user.surname, hash=h, email=user.email),
                         user_id)
    if user_n:
        return schemas.User(uid=user_n.uid, name=user_n.name, surname=user_n.surname, email=user_n.email)
    raise HTTPException(404, "Not found")


@router.delete("/{user_id}", tags=["users"])
async def remove_user_(user_id: int, db: Session = Depends(get_db),
                       current_user: models.User = Depends(get_current_user)):
    """Allows admin to remove users. All of their summaries will be given to admin."""
    if not check_admin(current_user) or current_user.uid==user_id:
        raise HTTPException(403, "You are not authorized.")
    u: models.User = get_user(db, user_id)
    if not u:
        raise HTTPException(404, "Not found")
    for s in u.summaries:
        s: models.Summary
        s.author_id = current_user.uid
    db.commit()
    db.delete(u)
    db.commit()
    return HTTPException(204, "User removed.")



from fastapi import APIRouter, Depends, Request
from erre2.dependencies import get_auth_token, get_db
from erre2.authentication import get_current_user
from erre2.database.crud import get_course, get_courses, create_course, update_course
from sqlalchemy.orm import Session
from erre2.database import schemas, models
from typing import Optional
import bcrypt

router = APIRouter(
    prefix="/course",
    tags=["courses"],
    responses={404: {"description": "Not found"}}
)


@router.get("/", tags=["courses"], response_model=schemas.CourseList)
async def read_course_list(request: Request, course_id: Optional[int] = None, db: Session = Depends(get_db)):
    """
    Returns list of all courses, or just one.
    """
    if not course_id:
        courses = get_courses(db)
        return schemas.CourseList(courses=courses)
    else:
        course = get_course(db, course_id)
        return schemas.CourseList(courses=[course, ])


@router.patch("/{course_id}", response_model=schemas.Course, tags=["courses"])
async def patch_course(update: schemas.Course, course_id: int, db: Session = Depends(get_db),
                       current_user: models.User = Depends(get_current_user)):
    """
    Allows course patching
    """
    c = update_course(db, update, course_id)
    return schemas.Course(cid=c.cid, name=c.name, professor=c.professor, curriculum=c.curriculum, year=c.year,
                          semester=c.semester)


@router.post("/", tags=["courses"], response_model=schemas.Course)
async def create_course_(request: Request, course: schemas.Course, db: Session = Depends(get_db),
                         current_user: models.User = Depends(get_current_user)):
    """
    Allows the creation of a new course
    """
    course: models.Course = create_course(db, schemas.Course(name=course.name, professor=course.professor,
                                                             curriculum=course.curriculum, year=course.year,
                                                             semester=course.semester))
    if course:
        return schemas.Course(name=course.name, professor=course.professor, curriculum=course.curriculum,
                              year=course.year, semester=course.semester)



from fastapi import APIRouter, Depends, Request, File, UploadFile, Form, HTTPException
from erre2.dependencies import get_auth_token, get_db, save_file
from erre2.authentication import get_current_user, check_admin
from erre2.database.crud import get_summaries, get_summary, create_summary, update_summary, get_summaries_course, \
    remove_summary
from sqlalchemy.orm import Session
from erre2.database import schemas, models
from typing import Optional
import bcrypt

router = APIRouter(
    prefix="/summary",
    tags=["summaries"],
    responses={404: {"description": "Not found"}, 204: {"description": "Removed"}}
)


@router.get("/", tags=["summaries"], response_model=schemas.SummaryList)
async def read_summary_list(request: Request, course_id: Optional[int] = None, db: Session = Depends(get_db)):
    """
    Returns list of all summaries, or just one.
    """
    if not course_id:
        summaries = get_summaries(db)
        return schemas.SummaryList(summaries=summaries)
    summaries = get_summaries_course(db, course_id)
    return schemas.SummaryList(summaries=summaries)


@router.patch("/{summary_id}", tags=["summaries"], response_model=schemas.Summary)
async def patch_summary(summary_id: int, update: str = Form(...), file: UploadFile = File(...),
                        db: Session = Depends(get_db), current_user: models.User = Depends(get_current_user)):
    """
    Allows summary patching
    {"summary":{"sid":0,"author_id":2,"course_id":1,"name":"Pippo",
    "filename":"string","downloads":0},"description":"Ciao"}
    """
    s = get_summary(db, summary_id)
    if not s:
        raise HTTPException(404, "Not found.")
    update: schemas.SummaryUpdate = schemas.SummaryUpdate.parse_raw(update)
    filename = await save_file(file, update.summary)
    s: schemas.Summary = update_summary(db, update.summary, summary_id, update.description, file)
    return s


@router.put("/{summary_id}")
async def put_summary(summary_id: int, update: schemas.SummaryName, db: Session = Depends(get_db),
                      current_user: models.User = Depends(get_current_user)):
    """
    Allows summary name to be updated
    """
    s: models.Summary = get_summary(db, summary_id)
    if not s:
        raise HTTPException(404, "Not found.")
    s.name = update.name
    db.commit()
    db.refresh(s)
    return s


@router.delete("/{summary_id}")
async def delete_summary(summary_id: int, db: Session = Depends(get_db),
                         current_user: models.User = Depends(get_current_user)):
    """
    Allows the removal of a summary
    """
    s: models.Summary = get_summary(db, summary_id)
    if not s:
        raise HTTPException(404, "Not found.")
    if s.author_id != current_user.uid and not check_admin(current_user):
        raise HTTPException(403, "Only the owner can delete the summary.")
    remove_summary(db, summary_id)
    return HTTPException(204, "Removed with success")


@router.post("/", tags=["summaries"], response_model=schemas.Summary)
async def create_summary_(summary: str = Form(...), file: UploadFile = File(...),
                          current_user: models.User = Depends(get_current_user), db: Session = Depends(get_db)):
    """
    Allows the creation of a new summary
    {"sid": 0,"author_id": 2,"course_id": 1,"name": "Pippo","filename": "string","downloads": 0}
    """
    summary = schemas.Summary.parse_raw(summary)
    s: schemas.Summary = create_summary(db, summary, file)
    await save_file(file, s)
    return s


@router.get("/download/{summary_id}", tags=["summaries"], response_model=schemas.Summary)
async def download_summary(summary_id: int, db: Session = Depends(get_db)):
    """
    Allows the download of a summary
    """
    s = get_summary(db, summary_id)
    if not s:
        raise HTTPException(status_code=404, detail="Couldn't find summary")
    s.downloads += 1
    db.commit()
    return s



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['erre2', 'erre2.database', 'erre2.routers']

package_data = \
{'': ['*'], 'erre2': ['Files/*']}

install_requires = \
['SQLAlchemy>=1.4.19,<2.0.0',
 'aiofiles>=0.7.0,<0.8.0',
 'bcrypt>=3.2.0,<4.0.0',
 'fastapi>=0.65.2,<0.66.0',
 'passlib[bcrypt]>=1.7.4,<2.0.0',
 'psycopg2>=2.9.1,<3.0.0',
 'pytest>=6.2.5,<7.0.0',
 'python-jose[cryptography]>=3.3.0,<4.0.0',
 'python-multipart>=0.0.5,<0.0.6',
 'requests>=2.26.0,<3.0.0',
 'uvicorn[standard]>=0.14.0,<0.15.0']

setup_kwargs = {
    'name': 'erre2',
    'version': '2.0.1',
    'description': 'A new and improved version of Erre2.',
    'long_description': None,
    'author': 'Lorenzo Balugani',
    'author_email': 'lorenzo.balugani@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.5,<4.0.0',
}


setup(**setup_kwargs)



from jose import JWTError, jwt
from pydantic import BaseModel
from typing import Optional
from fastapi.security import OAuth2PasswordBearer, OAuth2PasswordRequestForm
from fastapi import Depends, HTTPException, status
import bcrypt
from erre2.database import models
from sqlalchemy.orm import Session
from erre2.dependencies import get_db, SessionLocal
from erre2.database.crud import get_user_by_email, get_server
from datetime import datetime, timedelta
from passlib.context import CryptContext
from erre2.database.db import SessionLocal

SECRET_KEY = "supersecret"
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 120


class Token(BaseModel):
    access_token: str
    token_type: str


class TokenData(BaseModel):
    email: Optional[str] = None


oauth2_scheme = OAuth2PasswordBearer(tokenUrl="token")
pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")


def check_password(h, password):
    return pwd_context.verify(password, h)


def get_hash(password):
    return pwd_context.hash(password)


def authenticate_user(email: str, password: str):
    with SessionLocal() as db:
        user: models.User = get_user_by_email(db, email)
        if not user:
            return False
        if not check_password(user.password, password):
            return False
        return user


def create_token(data: dict):
    encode = data.copy()
    encode.update({"exp": datetime.utcnow() + timedelta(ACCESS_TOKEN_EXPIRE_MINUTES)})
    return jwt.encode(encode, SECRET_KEY, ALGORITHM)


def get_current_user(token: str = Depends(oauth2_scheme)):
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Could not validate credentials",
        headers={"WWW-Authenticate": "Bearer"},
    )
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        email: str = payload.get("sub")
        if email is None:
            raise credentials_exception
        token_data = TokenData(email=email)
    except JWTError:
        raise credentials_exception
    with SessionLocal() as db:
        user = get_user_by_email(db, token_data.email)
        if user is None:
            raise credentials_exception
        return user


def check_admin(user):
    with SessionLocal() as db:
        server = get_server(db)
        if server.owner_id!=user.uid:
            return False
        return True






from typing import List, Optional
from datetime import datetime

from pydantic import BaseModel


class User(BaseModel):
    uid: Optional[int]
    name: str
    surname: str
    email: str

    class Config:
        orm_mode = True


class UserList(BaseModel):
    users: List[User]


class UserCreatePlain(User):
    password: str

    class Config:
        orm_mode = True


class UserCreate(User):
    hash: bytes

    class Config:
        orm_mode = True


class Course(BaseModel):
    cid: Optional[int]
    name: str
    professor: str
    curriculum: str
    year: int
    semester: int

    class Config:
        orm_mode = True


class CourseList(BaseModel):
    courses: List[Course]

    class Config:
        orm_mode = True


class Commit(BaseModel):
    cid: Optional[int]
    description: str
    date: datetime
    summary_id: int

    class Config:
        orm_mode = True


class Summary(BaseModel):
    sid: Optional[int]
    author_id: int
    course_id: int
    name: str
    filename: str
    downloads: int
    author: Optional[User]
    course: Optional[Course]
    commits: Optional[List[Commit]]

    class Config:
        orm_mode = True


class SummaryList(BaseModel):
    summaries: List[Summary]

    class Config:
        orm_mode = True


class SummaryUpdate(BaseModel):
    summary: Summary
    description: str

    class Config:
        orm_mode = True


class SummaryName(BaseModel):
    name: str


class Server(BaseModel):
    name: str
    university: str
    monetization_link: str
    motd: str
    owner_id: int
    owner: Optional[User]

    class Config:
        orm_mode = True


class Planetarium(BaseModel):
    server: Server
    version: str
    type: str

    class Config:
        orm_mode = True
